"""
Memberships Model
"""

from memberaudit.models import CharacterSkillSetCheck, SkillSet

from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy as _

from squads.hooks import get_extension_logger
from squads.models.groups import Groups

logger = get_extension_logger(__name__)


class BaseFilter(models.Model):
    """BaseFilter Model store BaseFilter Data."""

    description = models.CharField(
        max_length=500,
        help_text=_("Filter description shown to Users."),
    )

    def __str__(self) -> str:
        return str(self.description)

    class Meta:
        abstract = True


class SkillSetFilter(BaseFilter):
    """SkillSetFilter Model store SkillSetFilter Data."""

    skill_sets = models.ManyToManyField(
        SkillSet,
        help_text=_(
            "Users must possess the selected skill sets with at least <strong>one</strong> character."
        ),
        related_name="squads_skill_sets",
    )

    def check_skill(self, user: User):
        """Check if user Characters has required skills."""
        qs = CharacterSkillSetCheck.objects.filter(
            character__eve_character__character_ownership__user=user,
            skill_set__in=list(self.skill_sets.all()),
            failed_required_skills__isnull=True,
        )
        return qs.exists()


class GroupSkillFilter(models.Model):
    """GroupFilter Model store Filter Data."""

    group = models.OneToOneField(Groups, on_delete=models.CASCADE)
    description = models.CharField(max_length=500, default="", blank=True)
    skill_filters = models.ManyToManyField(SkillSetFilter)
    enabled = models.BooleanField(default=True)  # Not Implemented yet

    def __str__(self) -> str:
        return str(f"Skill Requirment for {self.group.name}")

    class Meta:
        default_permissions = ()



"""Admin models"""

from typing import Any

from django.contrib import admin
from django.db.models.query import QuerySet
from django.http.request import HttpRequest

from squads.models.skills import GroupSkillFilter, SkillSetFilter


@admin.register(SkillSetFilter)
class SkillSetFilterAdmin(admin.ModelAdmin):
    """
    SkillSetFilterAdmin
    """

    list_display = ("description", "_skill_sets")
    filter_horizontal = ("skill_sets",)

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        qs = super().get_queryset(request)

        return qs.prefetch_related("skill_sets")

    @admin.display()
    def _skill_sets(self, obj) -> str:
        objs = obj.skill_sets.all()

        return ", ".join(sorted([obj.name for obj in objs]))


@admin.register(GroupSkillFilter)
class GroupFilterAdmin(admin.ModelAdmin):
    filter_horizontal = ["skill_filters"]
    list_display = ["__str__", "enabled"]



"""Groups views."""

from django.contrib.auth.decorators import login_required
from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
from django.shortcuts import get_object_or_404, redirect, render

from squads.forms import CommentForm
from squads.hooks import get_extension_logger
from squads.models.groups import Groups, Pending
from squads.models.memberships import Memberships
from squads.models.skills import GroupSkillFilter

logger = get_extension_logger(__name__)


@login_required
def broswe_groups(request):
    groups_list = Groups.objects.all()
    membership_ids = Memberships.objects.filter(user=request.user).values_list(
        "group_id", flat=True
    )
    paginator = Paginator(groups_list, 50)

    page = request.GET.get("page")
    try:
        groups = paginator.page(page)
    except PageNotAnInteger:
        groups = paginator.page(1)
    except EmptyPage:
        groups = paginator.page(paginator.num_pages)

    context = {
        "groups": groups,
        "membership_ids": membership_ids,
    }

    return render(request, "squads/groups/browse_groups.html", context)


@login_required
def squads_groups(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    return render(request, "squads/group_detail.html", {"group": group})


@login_required
def view_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    comment_form = CommentForm(request.POST or None)
    pending_application = Pending.objects.filter(group=group, user=request.user).first()
    membership = Memberships.objects.filter(user=request.user, group=group).first()

    req_skills = GroupSkillFilter.objects.filter(group=group)
    skill_req = True

    # TODO make a helper function to get missing skills displaying to view
    # Check if any of the required skills are missing
    for skill in req_skills:
        if not any(
            skill_filter.check_skill(request.user)
            for skill_filter in skill.skill_filters.all()
        ):
            skill_req = False
            break
    logger.debug("Skill Req: %s", skill_req)

    if request.method == "POST" and "join_group" in request.POST:
        return redirect("squads:view_group", group_id=group_id)

    return render(
        request,
        "squads/groups/view_group.html",
        {
            "group": group,
            "comment_form": comment_form,
            "pending_application": pending_application,
            "membership": membership,
            "skill_req": skill_req,
        },
    )



# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render

from squads.models.groups import Pending
from squads.models.memberships import Memberships


@login_required
@permission_required("squads.basic_access")
def squads_index(request):
    return render(request, "squads/index.html")


@login_required
@permission_required("squads.basic_access")
def squads_membership(request):
    memberships = Memberships.objects.filter(user=request.user)
    return render(
        request, "squads/groups/membership_groups.html", {"memberships": memberships}
    )


@login_required
def squads_pending(request):
    pending_memberships = Pending.objects.filter(user=request.user, approved=False)
    return render(
        request,
        "squads/groups/pending_groups.html",
        {"pending_memberships": pending_memberships},
    )



"""
Groups Model
"""

from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from allianceauth.authentication.models import User

from squads.hooks import get_extension_logger
from squads.managers import GroupsManager
from squads.view_helpers.core import generate_unique_id

logger = get_extension_logger(__name__)


class Groups(models.Model):
    """Groups Model store Groups Data."""

    name = models.CharField(max_length=255, unique=True)
    owner = models.ForeignKey(
        User, on_delete=models.CASCADE, related_name="group_owner"
    )
    description = models.TextField(blank=True)
    is_active = models.BooleanField(default=True)
    require_approval = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    image = models.ImageField(upload_to="squads/groups_images/", blank=True, null=True)

    objects = GroupsManager()

    def __str__(self) -> str:
        return str(self.name)

    def safe_description(self):
        return mark_safe(self.description)

    class Meta:
        default_permissions = ()
        permissions = (("squad_manager", "Can Manage Squads."),)


class Pending(models.Model):
    """Pending Model store Groups Data."""

    group = models.ForeignKey(Groups, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    approved = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    application = models.TextField(blank=True)
    application_id = models.CharField(
        max_length=12, primary_key=True, default=generate_unique_id, editable=False
    )

    class Meta:
        default_permissions = ()
